<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) 2019 Amasty (https://www.amasty.com)
 * @package Amasty_Coupons
 */


namespace Amasty\Coupons\Observer;

use Magento\Framework\Event\ObserverInterface;
use Amasty\Coupons\Model\DiscountCollector;
use Magento\Framework\Event\Observer;

/**
 * Class CouponObserver
 */
class CouponObserver implements ObserverInterface
{
    /**
     * @var DiscountCollector
     */
    protected $discountCollector;

    /**
     * @var string
     */
    private $codeKey;

    public function __construct(
        DiscountCollector $discountCollector
    ) {
        $this->discountCollector = $discountCollector;
    }

    /**
     * event salesrule_validator_process
     * @param Observer $observer
     */
    public function execute(Observer $observer)
    {
        $appliedCodes = [];

        if ($observer->getData('quote')->getCouponCode()) {
            $appliedCodes = explode(',', $observer->getData('quote')->getCouponCode());
        }

        if (!$this->codeKey) {
            $this->codeKey = 0;
        }
        
        if (!isset($appliedCodes[$this->codeKey])) {
            return;
        }
    /*
        if ($observer->getData('rule')->getCouponCode() != null) {
            $this->discountCollector->applyRuleAmount(
                $appliedCodes[$this->codeKey],
                $observer->getData('result')->getAmount()
            );
            $this->codeKey++;
        } elseif ($observer->getData('rule')->getUseAutoGeneration()) {
            $this->discountCollector->applyRuleAmount(
                $appliedCodes[$this->codeKey],
                $observer->getData('result')->getAmount()
            );
            $this->codeKey++;
        }
    */
        //reload all coupons and calculate discount
        $amount = 0;
        $observer->getResult()->setAmount($amount);
        $ruleId = $observer->getRule()->getRowId();
        $baseGrandTotal=$observer->getAddress()->getOrigData('subtotal_incl_tax');
        foreach ($appliedCodes as $coupon)
        {
            $discountData = $this->getDiscountByCoupon($coupon,$ruleId);
            $amount += $this->applyDiscount($discountData['simpleAction'],$discountData['discountAmount'],$baseGrandTotal);
        }
        $this->discountCollector->applyRuleAmount($this->getRuleDecription($ruleId),0);
        $observer->getResult()->setAmount($amount);

    }


    /**
     * @param $coupon
     * @param $CouponRuleId
     * @return array
     */
    public function getDiscountByCoupon($coupon,$CouponRuleId)
    {
        $discount = 0;
        try
        {
            $ruleId =   $this->coupon->loadByCode($coupon)->getRuleId();
            if($CouponRuleId == $ruleId)
            {
                $rule = $this->saleRule->load($ruleId);
                return ['discountAmount' => $rule->getDiscountAmount() ?? 0,"simpleAction" => $rule->getSimpleAction() ?? '', "description" => $rule->getDescription() ?? $coupon];
            }
            else
            {
                return ['discountAmount' => 0,"simpleAction" => '', "description"=>''];
            }

        }
        catch (\Exception $e)
        {
            $this->messageManager->addError("getDiscountDataByCoupon Error: $e");
        }

    }

    /**
     * @param $ruleId
     * @return string
     */
    public function getRuleDecription($ruleId)
    {
        if(isset($ruleId))
        {
            $rule = $this->saleRule->load($ruleId);
            return $rule->getDescription();
        }

    }


    /**
     * @param $simpleAction
     * @param $amount
     * @param $baseGrandTotal
     * @return float|int
     */
    public function applyDiscount($simpleAction,$amount,$baseGrandTotal)
    {
        $discount = 0;
        switch ($simpleAction)
        {
            case "by_percent":
                $discount = $baseGrandTotal*$amount/100;
                break;
            case "cart_fixed":
                $discount = $amount;
                break;
        }
        return $discount;
    }

}
