<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) 2019 Amasty (https://www.amasty.com)
 * @package Amasty_Coupons
 */


namespace Amasty\Coupons\Plugin;

use Amasty\Coupons\Model\CouponRenderer;
use Magento\SalesRule\Model\ResourceModel\Rule\Collection;
use Magento\Quote\Model\Quote\Address;
use Magento\SalesRule\Model\Coupon;
use Magento\Framework\App\ProductMetadataInterface;

/**
 * Class RuleCollection
 */
class RuleCollection
{
    /**
     * @var CouponRenderer
     */
    private $couponRenderer;

    /**
     * @var string
     */
    private $couponCode;

    /**
     * @var bool
     */
    private $isNeedToReplace = false;

    /**
     * @var Coupon
     */
    private $coupon;

    /**
     * @var ProductMetadataInterface
     */
    private $productMetadata;

    public function __construct(
        CouponRenderer $couponRenderer,
        Coupon $coupon,
        ProductMetadataInterface $productMetadata
    ) {
        $this->couponRenderer = $couponRenderer;
        $this->coupon = $coupon;
        $this->productMetadata = $productMetadata;
    }

    /**
     * @param Collection $subject
     * @param int $websiteId
     * @param int $customerGroupId
     * @param string $couponCode
     * @param string|null $now
     * @param Address $address
     *
     * @return array|null
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function beforeSetValidationFilter(
        Collection $subject,
        $websiteId,
        $customerGroupId,
        $couponCode = '',
        $now = null,
        $address = null
    ) {
        $this->isNeedToReplace = false;
        if (!is_string($couponCode) || strpos($couponCode, ',') === false) {
            return null;
        }
        $this->couponCode = $couponCode;
        $couponCode = $this->couponRenderer->render($couponCode);
        if (is_string($couponCode)) {
            return [$websiteId, $customerGroupId, $couponCode, $now, $address];
        }
        $this->isNeedToReplace = true;
        return null;
    }

    /**
     * @param Collection $subject
     * @param Collection $result
     *
     * @return Collection
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function afterSetValidationFilter(Collection $subject, $result)
    {
        if ($this->isNeedToReplace === false) {
            return $result;
        }

        $ruleIds = [];
        $isVersionCompare = version_compare($this->productMetadata->getVersion(), '2.2.8', '<');

        $coupons = $this->couponRenderer->parseCoupon($this->couponCode);

        if (!$isVersionCompare) {
            foreach ($coupons as $coupon) {
                $ruleIds[] = $this->coupon->loadByCode($coupon)->getRuleId();
            }
        }

        $connection = $result->getConnection();
        $select = $result->getSelect();
        $search = $connection->quoteInto(
            'code = ?',
            $this->couponCode
        );
        $replace = $connection->quoteInto(
            'code IN (?)',
            $coupons
        );

        $wherePart = $select->getPart(\Zend_Db_Select::WHERE);

        foreach ($wherePart as &$where) {
            if ($isVersionCompare) {
                $where = str_ireplace($search, $replace, $where);
            } else {
                $replace = $connection->quoteInto(
                    'rule_id IN (?)',
                    $ruleIds
                );
                $where = str_ireplace('rule_id IN (NULL)', $replace, $where);
            }
        }

        $select->setPart(\Zend_Db_Select::WHERE, $wherePart);
        $select->group('rule_id');

        return $result;
    }
}
